<?php
//
// phpIpacstats - A web frontend for ipac[-ng]
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// $Id: index.php,v 1.31 2002/12/10 16:06:56 bene1 Exp $
// file: index.php
// function: main script


//retrieve settings from config-file
require('config.php');

//set program version that is displayed in footer
$prog_version='0.5';

// default to english, but this is negotiable.
if (!(isset($lng) && file_exists('includes/lang/' . $lng . '.php'))) {
    $lng = $default_lang;
}

require('includes/lang/' . $lng . '.php');  // get our language include
require('includes/class.Template.inc.php');  // template library
require('includes/system_functions.php');    // finish defining all of our global functions and variables
require('includes/parse_output.php');

// figure out if we got a template passed in the url
if (!(isset($template) && file_exists("templates/$template"))) {
    // default template we should use if we don't get a argument.
    define('TEMPLATE_SET', $default_template);
    $template = $default_template;
} else {
    define('TEMPLATE_SET', $template);
}

// arrays where we will save occured errors

$sys_errors = array();
$input_errors = array();

// figure out how to do the scaling
if (!isset($Scale)) {
    switch ($img_normalize) {
	case('8'):  
	            $Scale = "bit_per_sec";
		    break;
	default:
	case('1'):
	            $Scale = "bytes_per_sec";
		    break;
    }
}
switch ($Scale) {
    case('bit_per_sec'):  
	$img_normalize = '8';
	break;
    default:
    case('bytes_per_sec'):
	$img_normalize = '1';
	break;
}


// figure out how the fixed quantity for the report
if (!isset($Quantity)) {
	switch ($report_quantity) {
	case('TB'):  
	            $Quantity = "terabytes";
		    break;
	case('GB'):  
	            $Quantity = "gigabytes";
		    break;
	case('MB'):  
	            $Quantity = "megabytes";
		    break;
	case('KB'):  
	            $Quantity = "kilobytes";
		    break;
	default:
	case('B'):  
	            $Quantity = "bytes";
		    break;
    }
}
switch ($Quantity) {
	case('terabytes'):  
			$report_quantity = 'T';
			break;
	case('gigabytes'):  
			$report_quantity = 'G';
			break;
	case('megabytes'):  
			$report_quantity = 'M';
			break;
	case('kilobytes'):  
			$report_quantity = 'K';
			break;
	default:
	case('bytes'):
			$report_quantity = '';
			break;
}


// figure out if we got times or dates passed
if (!(isset($Start_Date))) {
    $Start_Date = 'YYYY/MM/DD';
}
if (!isset($Start_Time)) {
    $Start_Time = 'hh:mm:ss';
}
if (!isset($End_Date)) {
    $End_Date = date("Y/m/d");
}
if (!isset($End_Time)) {
    $End_Time = date("H:i:s");
}
if (!isset($Time_Method)) {
    $Time_Method = "Pre_Frame";
}

//split up date and time-strings into arrays
if ($Time_Method == "Abs") {
   $startdate_ar = explode("/",$Start_Date);
   $starttime_ar = explode(":",$Start_Time);
   $enddate_ar = explode("/",$End_Date);
   $endtime_ar = explode(":",$End_Time);

    // do some input data checks

    require('includes/inputcheck.php'); 
    
}


// we need ipac.conf to proceed, so see if we can read it
if (!is_readable($ipac_conffile)){
    array_push($sys_errors, $text["ipac_conf_not_readable"]);
}

// see if ipacsum path is set correctly and ipacsum is executable
if (!is_executable($ipacsum_executable)) {
    array_push($sys_errors, $text["ipacsum_not_exec"]);
}

if(!exec("$ipacsum_executable --help --png | grep 'This machine does have it'") ) {
    array_push($sys_errors, $text["no_perl_gdlib"]);
}

$radiobuttons = array();

if (count($input_errors) != 0 or count($sys_errors) != 0) {
    $text['title'] .= ": ".$text['error']."!";
    switch ($Time_Method) {
	case('Abs'):  
	            $radiobuttons['Abs'] = 'checked';
		    break;
	default:
	case('Pre_Frame'):
		    $radiobuttons['Pre_Frame'] = 'checked';
		    break;
    }
} else {
    switch($Time_Method) {
	case ('Abs') :
			$radiobuttons['Abs'] = 'checked';
			
			// add leading '0' to dates/times if neccessary
			
			reset ($startdate_ar);
			next($startdate_ar);
			array_walk($startdate_ar, 'addzero');
			reset($starttime_ar);
			array_walk($starttime_ar, 'addzero');
			reset($enddate_ar);
			next($enddate_ar);
			array_walk($enddate_ar, 'addzero');
			reset($endtime_ar);
			array_walk($endtime_ar, 'addzero');
			
			$timestring = "-s $startdate_ar[0]$startdate_ar[1]$startdate_ar[2]"
				    . "$starttime_ar[0]$starttime_ar[1]$starttime_ar[2]"
				    . "  -e $enddate_ar[0]$enddate_ar[1]$enddate_ar[2]"
				    . "$endtime_ar[0]$endtime_ar[1]$endtime_ar[2]";
			break;
	case ('Pre_Frame'):
	default:	$radiobuttons['Pre_Frame'] = 'checked';
			if (!isset($timeframe)) {
			    $timeframe = 'today';
			}
			$text['title'] .= ": ".$text[$timeframe];
			$timestring = "-t \"$timeframe\"";
    }
}

// adds leading '0' if neccessary
function addzero(&$value, $key) {
    if (ereg("^[0-9]$",$value)) {
	$value = "0".$value;
    }
}


// fire up the template engine
$tpl = new Template(dirname(__FILE__) . '/templates/' . TEMPLATE_SET);
$tpl->set_file(array(
    'form' => 'form.tpl'
));


// print out a box of information
function makebox ($title, $content)
{
    $t = new Template(dirname(__FILE__) . '/templates/' . TEMPLATE_SET);

    $t->set_file(array(
        'box'  => 'box.tpl'
    ));

    $t->set_var('title', $title);
    $t->set_var('content', $content);

    return $t->fp('out', 'box');
}  



// if error occured, print error page and stop
if (count($input_errors) != 0 or count($sys_errors) !=0) {

    // print error messages for system type errors
    if (count($sys_errors) != 0) {
	$errormsg .= <<<EOD
	    <p class="error">{$text[sys_error_occured]}
	    <ul>
EOD;
	while(list($key,$val) = each($sys_errors)) {
	    
	    $errormsg .= "<li> $val";
	}
	$errormsg .= "</ul>";
    } else 
    
    // print error messages for input type errors
    if (count($input_errors) != 0) {
	$errormsg .= <<<EOD
	    <p class="error">{$text[an_error_occured]}
	    <ul>
EOD;
	while(list($key,$val) = each($input_errors)) {
	    
	    $errormsg .= "<li> $val";
	}
	$errormsg .= "</ul>";
    }
    
    
    // let the page begin...

    require('includes/system_header.php');
    $tpl->set_var('title', $text['title']);

    $tpl->set_var('textstats', makebox($text['error'], $errormsg));

    //ok let's show the images...
    //$tpl->set_var('input', makebox($text['input'], ""));

    //$tpl->set_var('output', makebox($text['output'], ""));

    require('includes/configuration.php');

    require('includes/about.php');

    // parse our the template
    $tpl->pfp('out', 'form');

    print "</body></html>";
    exit();

}

//okay, no errors, now hurry on...

//get rulenames and seperate in/out-coming and "both" rules

//print "<pre>";
/*
$in_rules = array();	    //rule-names of incoming rules saved here
$out_rules = array();	    //rule-names of outgoing rules saved here
$in_files = array();	    //filenames of the images of incoming rules
$out_files = array();	    // -- same for outgoing
*/

$content = array(
	    "rules" => array(),
	    "in" => array(),
	    "out" => array()
	   );

if ($fp = fopen($ipac_conffile, 'r')) {
    while (!feof($fp)) {
        $row = fgets($fp, 4096);
        if (($row[0]!='#') && ($row[0]!='%')) {
            unset($fields);
            ereg("^([a-zA-Z0-9\\_\. <>-]+)\|([a-zA-Z0-9_\~ -]+)\|",$row,$fields);
            
            // take care of new style rule names
            $fields[2] = preg_replace("/^.*\~fi$/","in",$fields[2]);
            $fields[2] = preg_replace("/^.*\~i$/","out",$fields[2]);
            $fields[2] = preg_replace("/^.*\~fo$/","out",$fields[2]);
            $fields[2] = preg_replace("/^.*\~o$/","in",$fields[2]);
            
            // as of ipac-ng (1.23 and above) rules can be very complex
            // containing both 'in' and 'out' rules so we obey the fact
            // that the user names a rule ".... in" or ".... out"
            if (preg_match("/^.* in$/",$fields[1]))
                $fields[2] = 'in';
            if (preg_match("/^.* out$/",$fields[1]))
                $fields[2] = 'out';
            
            if ($fields[2]=='in' or $fields[2]=='out') {
                // don't insert a rule more than once, those rules 
                // get summarized by ipacsum anyway
                if (!in_array($fields[1], $content["rules"], TRUE)) {
                    array_push($content["rules"], $fields[1]);
                    array_push($content[$fields[2]], array(
                        "rule" => $fields[1],
                        "file" => good_filename($fields[1])
                    ));
                }
            }
        } 
    }
    fclose($fp);
} else { print "error opening conffile"; }

// find out which filename ipacsum uses for this rule
function good_filename ($rulename) {
    return preg_replace("/[\s\\/\|\>\<]/","_",$rulename);
}

$ipac_args=$timestring." --png $data_dir --png-average-curve 15 ";
$ipac_args.=' --png-height '.$img_height.' --png-width '.$img_width;
$ipac_args.=' --png-normalize '.$img_normalize;
$ipac_args.=' --fixed-quantity \''.$report_quantity.'\'';

//call the ipacsum-program

if ($fp = popen("$ipacsum_executable $ipac_args", 'r')) {
       while (!feof($fp)) {
          $execreturn .= fgets($fp, 4096);
       }
       $execreturn=trim($execreturn);
}

// only show images if ipacsum created some output

if ($execreturn == "") {
    $textstats = <<<EOD
<br><br>
<div align="center">
<b>{$text['no_output']}</b>
</div>
<br><br>
EOD;
} else {
    $textstats = parse_output($execreturn, $content);
   
    // generate html-code for images on incoming-side
    $in_image_code = create_code($content['in']);

    //generate html-code for images on outgoing-side
    $out_image_code = create_code($content['out']);
}

//return html-code where all images are displayed
function create_code($content) {
    global $text,$img_width,$img_height,$img_mode,$data_dir;
    unset($code);
    $filename = $data_dir.'/'.$content[0]['file'].'.png';
    if (is_readable($filename)) {
	$extension = "png";
    } else {
	$extension = "gif";
    }
    for ($i=0;$i<count($content);$i++) {
        $code .= "	\n	<p>{$text[rule]}: ". htmlspecialchars($content[$i]['rule']).
	         "<br>	<div align=\"center\">";
	switch ($img_mode) {
	    case 'php': 
		$code .= '<img src="image.php?image='.$data_dir.'/'.$content[$i]['file'].'.'.
			 $extension.'" height="'.$img_height.'" ' .
		         'width="'.$img_width.'" alt="'.$text['stat_of_rule'].
			 ' '.$content[$i]['rule'].'">';
	    break;
	    case 'disk':
		$code .= '<img src="'.$data_dir.'/'.$content[$i]['file'].'.'.$extension.
			 '" height="'.$img_height.'" ' .
			 'width="'.$img_width.'" alt="'.$text['stat_of_rule'].
			 ' '.$content[$i]['rule'].'">';
	    break;
	}
	$code .= '</div>';
    }
    return $code;
}



// ************************************************************************ 
// let the page begin.

require('includes/system_header.php');
$tpl->set_var('title', $text['title']);

//print "<br>".$ipacsum_executable." ".$ipac_args."<br>";
//print $Time_Method;

$tpl->set_var('textstats', makebox($text['textstats'], $textstats));

//ok let's show the images...
if ($in_image_code != "") {
    $tpl->set_var('input', makebox($text['input'], $in_image_code));
}
if ($out_image_code != "") {
    $tpl->set_var('output', makebox($text['output'], $out_image_code));
}

require('includes/configuration.php');

require('includes/about.php');

// parse our the template
$tpl->pfp('out', 'form');

?>
</body>
</html>
